from typing import List, Tuple, Any, Optional
from pathlib import Path
import zipfile
from tempfile import NamedTemporaryFile
from tqdm import tqdm
import logging
import re
import ctypes
from os import sep
from varc_core.systems.base_system import BaseSystem

# based on https://stackoverflow.com/questions/48897687/why-does-the-syscall-process-vm-readv-sets-errno-to-success and PymemLinux library

class IOVec(ctypes.Structure):
    _fields_ = [
        ("iov_base", ctypes.c_void_p),
        ("iov_len", ctypes.c_size_t)
    ]

class LinuxSystem(BaseSystem):
    
    def __init__(
        self,
        include_memory: bool,
        include_open: bool,
        extract_dumps: bool,
        **kwargs: Any
    ) -> None:
        super().__init__(include_memory=include_memory, include_open=include_open, extract_dumps=extract_dumps, **kwargs)
        self.libc = ctypes.CDLL("libc.so.6")
        self.process_vm_readv = self.libc.process_vm_readv
        self.process_vm_readv.args = [ # type: ignore
            ctypes.c_int, 
            ctypes.POINTER(IOVec), 
            ctypes.c_ulong, 
            ctypes.POINTER(IOVec), 
            ctypes.c_ulong, 
            ctypes.c_ulong
        ]
        self.process_vm_readv.restype = ctypes.c_ssize_t
        if self.include_memory:
            self.dump_processes()
            if self.extract_dumps:
                from varc_core.utils import dumpfile_extraction
                dumpfile_extraction.extract_dumps(Path(self.output_path))

    def parse_mem_map(self, pid: int, p_name: str) -> List[Tuple[int, int]]:
        """Returns a list of (start address, end address) tuples of the regions of process memory that are mapped
        
        
        """
        map_addresses = []
        mem_map_path = Path(f"/proc/{pid}/maps")
        try:
            with mem_map_path.open(mode="r") as mem_map:
                map_content = mem_map.readlines()
                for line in map_content:
                    line_groups = re.match(r"([a-fA-F0-9]+)-([a-fA-F0-9]+)\s(r|-)" , line)
                    if line_groups.group(3) == "r": # type: ignore # Only collecting pages that are readable
                        page_start = int(line_groups.group(1), 16) # type: ignore
                        page_end = int(line_groups.group(2), 16) # type: ignore
                        map_addresses.append((page_start, page_end))
        except FileNotFoundError:
            logging.warning(f"Could not parse memory map for {p_name} (pid {pid}). Cannot dump this process.")
            return map_addresses
        except PermissionError:
            logging.warning(f"Permission denied parsing memory map for {p_name} (pid {pid}). Cannot dump this process.")
            return map_addresses

        return map_addresses

    def read_bytes(self, pid: int, address: int, byte: int) -> Optional[bytes]:
        """Reads {byte} bytes from the base memory address {address} in the virtual memory space of process {pid}
        
        :param pid: int of the process id
        :param address: int of the addrss
        :param byte: int of current bytre

        :return: Bytes from memory location
        :rtype: bytes
        """

        buff = ctypes.create_string_buffer(byte)
        io_dst = IOVec(ctypes.cast(ctypes.byref(buff), ctypes.c_void_p), byte)
        io_src = IOVec(ctypes.c_void_p(address), byte)

        linux_syscall = self.process_vm_readv(pid, ctypes.byref(io_dst), 1, ctypes.byref(io_src), 1, 0)

        if linux_syscall == -1:
            return None

        return buff.raw

    def dump_processes(self) -> None:
        """Dumps all processes to temp files, adds temp file to output archive then removes the temp file"""
        archive_out = self.output_path
        with zipfile.ZipFile(archive_out, "a", compression=zipfile.ZIP_DEFLATED) as zip_file:
            for proc in tqdm(self.process_info, desc="Process dump progess", unit=" procs"):
                pid = proc["Process ID"]
                p_name = proc["Name"]
                maps = self.parse_mem_map(pid, p_name)
                if not maps:
                    continue
                with NamedTemporaryFile(mode="w+b", buffering=0, delete=True) as tmpfile:
                    try:
                        for map in maps:
                            page_start = map[0]
                            page_len = map[1] - map[0]
                            mem_page_content = self.read_bytes(pid, page_start, page_len)
                            if mem_page_content:
                                tmpfile.write(mem_page_content)
                        zip_file.write(tmpfile.name, f"process_dumps{sep}{p_name}_{pid}.mem")
                    except PermissionError:
                        logging.warning(f"Permission denied opening process memory for {p_name} (pid {pid}). Cannot dump this process.")
                        continue
                    except OSError as oserror:
                        logging.warning(f"Error opening process memory page for {p_name} (pid {pid}). Error was {oserror}. Dump may be incomplete.")
                        pass
                    

        logging.info(f"Dumping processing has completed. Output file is located: {archive_out}")


import os
import re


def remove_special_characters(string: str) -> str:
    """Removes special characters from string.

    :param string: string to modify
    :return: string without special chars
    """
    return re.sub('\W+', '', string)


def strip_drive(path: str) -> str:
    """Removes drive prefix from a path

    :param path: Path string to remove drive from

    :return: Windows path without drive prefix
    """
    new_path = os.path.splitdrive(path)[1]
    new_path = new_path[1:] if new_path.startswith(os.sep) else new_path
    return new_path



import unittest
from zipfile import ZipFile

from varc_core.systems import BaseSystem, acquire_system

class TestBaseCases(unittest.TestCase):
    system: BaseSystem
    zip_path: str

    @classmethod
    def setUpClass(cls) -> None:
        cls.system = acquire_system()
        cls.zip_path = cls.system.acquire_volatile()


    @classmethod
    def tearDownClass(cls) -> None:
        pass

    def test_some_processes(self) -> None:
        processes = self.system.get_processes()
        self.assertTrue(len(processes) > 0)
        process_names = [process["Name"] for process in processes]
        self.assertIn("python3", process_names)
        
        
    def test_dump_files(self) -> None:
        open_files = self.system.dump_loaded_files()
        self.assertTrue(len(open_files) > 0)
        # Check we pulled at least one file from /bin/
        with ZipFile(self.zip_path) as z:
            binary_files = [binary for binary in z.namelist() if ("/bin/" in binary in binary.lower())]
            self.assertGreater(len(binary_files), 0)
                        


"""VARC custom exceptions
"""

class VarcException(Exception):
    """Base class for exeptions with a message that can be displayed to users
    """

    def __init__(self, message: str) -> None:
        super().__init__(message)
        self.user_message = message

class MissingOperatingSystemInfo(VarcException):
    def __init__(self) -> None:
        super().__init__("Failed to detect operating system.")



"""This file carves out binary and log data from systems, heavily based on RipRaw
"""
import string
import os
import os.path
import shutil
from datetime import datetime
import logging
from typing import Union
from typing import List
import mimetypes
from tempfile import TemporaryDirectory
from pathlib import Path
from os import listdir
from os.path import isfile, join
import zipfile
import magic
import re


# Used to extract strings
ASCII_BYTE = " !\"#\$%&'\(\)\*\+,-\./0123456789:;<=>\?@ABCDEFGHIJKLMNOPQRSTUVWXYZ\[\]\^_`abcdefghijklmnopqrstuvwxyz\{\|\}\\\~\t"
n = 6
combined_reg = "(?:[%s]\x00|[%s]){%d,}" % (ASCII_BYTE, ASCII_BYTE, n)
combined_re = re.compile(combined_reg)
# Allow lines with 7+ chars
good_line = re.compile("[ 0-9a-zA-Z\.:]{7,}")

# First files in the list match first
# Used for file carving
file_markers = [
    # elf
    "7f 45 4c 46 02 01 01",
    # jpg
    "ff d8 ff e0",
    # 7z
    "37 7a bc af 27",
    # avi
    "41 56 49 20",
    # bz
    "42 5A 68",
    # docx
    "50 4b 03 04 14",
    # doc
    "d0 cf 11 e0 a1",
    # png
    "89 50 4e 47",
    # rar
    "52 61 72 21",
    # zip
    "50 4b 30 30",
    # exe
    "4d 5a 90 00 03",
    # 2021
    "30 32 31 2d",
    # 2022
    "30 32 32 2d",
    # ElfChnk EVT
    "45 6c 66 43 68 6e 6b",
    # Evtx chunk
    "2a 2a 00 00",
    # PNG
    "89 50 4e 47 0d 0a 1a 0a",
    # doc
    "d0 cf 11 e0 a1 b1",
    # pst
    "21 42 4e a5 6f b5 a6",
    # <html
    "3c 68 74 6d",
    # <HTML
    "3c 48 54 4d",
    # LNK File
    "4c 00 00 00 01 14 02 00 00 00 00 00 c0 00 00 00 00 00 00 46",
    # <plist
    "70 6c 69 73 74",
]


def combined_strings_text(buf: bytes) -> str:
    """ Get strings worth indexing """
    results = []
    buf = buf.replace(b"\xff", b"\x00")
    decoded_text = buf.decode("utf8", "ignore")
    lines = combined_re.findall(decoded_text)
    current_regex = good_line
    for line in lines:
        if current_regex.search(line):
            results.append(line)
    text_output = "\n".join(results)
    # Remove non printable
    text_output = "".join(filter(lambda x: x in string.printable, text_output))
    return text_output


def combined_strings(buf: bytes) -> int:
    """Returns how many strings are in the buffer

    :param 
    """
    return len(combined_strings_text(buf))


def write_file(file_count: int, data_bytes: bytes, output_dir: Path, output_prefix: str, text_mode: bool = False) -> None:
    """Write bytes to a file as text or binary

    :param file_count:
    :param data_bytes:
    :param output_dir:
    :param output_prefix:
    :param text_mode: 
    """

    file_extension = ".bin"
    if text_mode:
        file_extension = ".log"

    mime_type = magic.from_buffer(data_bytes, mime=True)

    if mime_type != "application/octet-stream":
        extension = mimetypes.guess_extension(mime_type)
        if extension:
            file_extension = extension

    file_name = output_prefix + str(file_count) + file_extension
    file_path = os.path.join(output_dir, file_name)

    if text_mode:
        text_content = combined_strings_text(data_bytes)

        # Sub-split the text/log file if it contains dates
        # Into a single file for each possible log entry
        now = datetime.utcnow()
        current_year = str(now.year)
        last_year = str(now.year - 1)
        if current_year in text_content or last_year in text_content:
            log_parts = []
            if current_year in text_content:
                log_parts = text_content.split(current_year)
            if last_year in text_content:
                log_parts = text_content.split(last_year)

            # Append delimeter year at the start of each split text
            file_extension = ".log"

            for split_count, part in enumerate(log_parts):
                text_part = part
                if split_count != 0:
                    text_part = current_year + part

                file_name = (
                    output_prefix
                    + str(file_count)
                    + "_"
                    + str(split_count)
                    + file_extension
                )
                file_path = os.path.join(output_dir, file_name)

                with open(file_path, "w") as f:
                    f.write(text_part)

        # Text but doesnt contain a date
        else:
            with open(file_path, "w") as f:
                f.write(str(data_bytes))

    else:
        # Cant find a way to stop mypy erroring
        with open(file_path, "wb") as f:  # type: ignore
            f.write(data_bytes)  # type: ignore


def split_buffer(buffer: bytes, start_text: bool, file_markers: List[str]) -> int:
    """Split into text and data halves

    :param buffer:
    :param start_text: 
    :param file_markers:

    :return:
    """
    for file_marker in file_markers:
        byte_marker = bytearray.fromhex(file_marker)

        if byte_marker in buffer:
            before = buffer.split(byte_marker)[0]
            return len(before)

    # No header matches - Split on text vs binary
    count = 0
    for b in buffer:
        is_text = str(chr(b)).isprintable()
        # If it started with text
        if start_text:
            if not is_text:
                # We've found the split point
                return count
        # If starts with binary
        else:
            if is_text:
                # We've found the split point
                return count
        count += 1
    return count


def zip_folder(dir_name: str) -> str:
    """Zip a folder
    
    :param dir_name: path to zip
    """
    if not os.path.exists(dir_name):
        os.mkdir(dir_name)
    return shutil.make_archive(dir_name, "zip", dir_name)


def extract_dumps(input_archive: Path) -> Union[str, None]:
    """Carve process memory dump for potentially useful embedded files
    
    :param input_archive: 
    """

    logging.info("Beginning process memory dump carving")

    # 10 MB max filesize - Increasing this will slow performance
    MAX_FILESIZE = 1024 * 10000
    # Amount to read each iteration
    READ_AMOUNT = 10240

    file_count = 0
    text_mode = False
    data_buffer = bytes()

    with zipfile.ZipFile(input_archive, "a", zipfile.ZIP_DEFLATED) as dump_archive:
        for proc_dump in dump_archive.namelist():
            if proc_dump.startswith("process_dumps") and proc_dump.endswith(".mem"):
                dump_file_name = proc_dump.split("/")[-1]
                output_prefix = dump_file_name.split(".")[0]
                logging.info(f"Carving process dump {dump_file_name}")
                with dump_archive.open(proc_dump, "r") as dump:
                    with TemporaryDirectory() as extract_dir:
                        while True:
                            data = dump.read(READ_AMOUNT)

                            if bytearray(READ_AMOUNT) == data:
                                # Skip empty sections
                                pass
                            else:
                                just_split = False
                                strings_length = combined_strings(data)
                                # TODO: Remove
                                split_point = 0

                                if text_mode:
                                    # We're now looking at strings
                                    if strings_length < 1000 or len(data_buffer) > MAX_FILESIZE:
                                        # logging.info("Processed " + str(mb_processed) + " Megabytes")
                                        # logging.info("Buffer length: " + str(len(data_buffer)))
                                        split_point = split_buffer(data, True, file_markers)
                                        text_mode = False
                                        file_count += 1
                                        data_buffer += data[:split_point]
                                        write_file(file_count, data_buffer, Path(extract_dir), output_prefix, True)
                                        data_buffer = data[split_point:]
                                        just_split = True

                                else:
                                    # Now we're looking at binary
                                    if strings_length >= 1000 or len(data_buffer) > MAX_FILESIZE:
                                        # logging.info("Processed " + str(mb_processed) + " Megabytes")
                                        # logging.info("Buffer length: " + str(len(data_buffer)))
                                        split_point = split_buffer(data, False, file_markers)
                                        text_mode = True
                                        file_count += 1
                                        write_file(file_count, data_buffer + data[:split_point], Path(extract_dir), output_prefix, False)
                                        data_buffer = data[split_point:]
                                        just_split = True

                                if not just_split:
                                    data_buffer += data

                            if len(data) < READ_AMOUNT:
                                #logging.info("Less than expected data, passing")
                                write_file(file_count, data_buffer + data[:split_point], Path(extract_dir), output_prefix, text_mode)
                                break

                        # Write each carved file into dir in zip
                        carved_files = [file for file in listdir(extract_dir) if isfile(join(extract_dir, file))]
                        for carved in carved_files:
                            carved_filepath = join(extract_dir, carved)
                            dump_archive.write(carved_filepath, f"{proc_dump}_carved/{carved}")

                logging.info("Carving of process dumps complete")
    return None



"""Every get_ function must return: List[dict]

notepad.exe.mem - Dumped memory
notepad.exe.mem.log - Dumped memory log

Try to keep functions working cross-platform where possible
If it can't work cross-platform, put any platform specific code in the class that inherits this base
    e.g. In linux.py
"""
import psutil
import socket
import os
import os.path
import json
import zipfile
from typing import List
from datetime import datetime
import logging
from typing import Optional
import mss
import time

from varc_core.utils.string_manips import remove_special_characters, strip_drive


_MAX_OPEN_FILE_SIZE = 10000000 # 10 Mb max dumped filesize


class BaseSystem:
    """A 

    :param process_name: 
    :param process_id: 
    :param take_screenshot: 
    :param include_memory: 
    :param include_open: 
    :param extract_dumps: 
    """
    def __init__(
        self,
        process_name: Optional[str] = None,
        process_id: Optional[int] = None,
        take_screenshot: bool = True,
        include_memory: bool = True,
        include_open: bool = True,
        extract_dumps: bool = False,
    ) -> None:
        self.todays_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        logging.info(f'Acquiring system: {self.get_machine_name()}, at {self.todays_date}')
        self.timestamp = datetime.timestamp(datetime.now())
        self.process_name = process_name
        self.process_id = process_id
        self.extract_dumps = extract_dumps
        self.include_memory = include_memory
        self.include_open = include_open
    
        if self.process_name and self.process_id:
            raise ValueError("Only one of Process name or Process ID (PID) can be used. Please re-run using one or the other.")
        self.zip_path = self.acquire_volatile()

    def get_network(self) -> List[str]:
        """Get active network connections
            
        :return: List of netsta logs
        :rtype List[string]
        """
        network = []
        try:
            connections = psutil.net_connections()
        except psutil.AccessDenied:
            logging.error("Access denied attempting to get network connections") # without sudo on osx
            connections = []
        for conn in connections:
            if conn.laddr and conn.raddr:
                syslog_date: str = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                process_name: str = psutil.Process(conn.pid).name()
                log_line = f"{syslog_date} {conn.laddr.ip} {conn.laddr.port} {conn.raddr.ip} {conn.raddr.port} {process_name}"
                network.append(log_line)
        return network

    def get_processes_dict(self) -> List[dict]:
        """Get processes on system, potentially filtered

        :return: List of processes as dicts
        """
        if self.process_id:
            return [psutil.Process(self.process_id).as_dict()]
        elif self.process_name:
            process_choice = []
            for proc in psutil.process_iter():
                proc_dict = proc.as_dict()
                if proc_dict["name"].lower() == self.process_name.lower():
                    process_choice.append(proc_dict)
            return process_choice
        else:
            return [proc.as_dict() for proc in psutil.process_iter()]

    def dump_loaded_files(self) -> List[str]:
        """Collects files that are open

        :return: List of filepaths that were collected
        """

        process_choice = self.get_processes_dict()

        open_files: List[str] = []
        mapped_filepaths: List[str] = []
        exe_paths: List[str] = []

        for process in process_choice:
            proc_open_files = process.get("open_files", [])
            if proc_open_files:
                open_files += [open_file.path for open_file in proc_open_files]
            proc_memory_maps = process.get("memory_maps", []) 
            if proc_memory_maps:
                mapped_filepaths += [path.path for path in proc_memory_maps]
            proc_exe = process.get("exe", [])
            if proc_exe:
                exe_paths += proc_exe

        # Combine and unique
        paths = list(set(open_files + mapped_filepaths + exe_paths))
        # only return paths that exist
        return [path for path in paths if (len(path) > 1 and os.path.exists(path) and os.path.getsize(path))]

    def get_processes(self) -> List[dict]:
        """Get running process(es) 

        :return: List of running processes - e.g. [{'pid': 1}]
        """
        process_data: List[dict] = []

        process_choice = self.get_processes_dict()

        for process in process_choice:
            creation_time = datetime.utcfromtimestamp(process["create_time"]).strftime('%Y-%m-%d %H:%M:%S')
            open_files_raw = process["open_files"]
            open_files = []
            if open_files_raw:
                for open_file in open_files_raw:
                    open_files.append(open_file.path)
            open_files_str = " ".join(open_files)
            cmd_line = ""
            # Windows
            if isinstance(process["cmdline"], str):
                cmd_line = process["cmdline"]
            # Linux, OSX
            if isinstance(process["cmdline"], List):
                cmd_line = " ".join(process["cmdline"])
            connections = []    
            if "connections" in process:
                if process["connections"]:
                    for conn in process["connections"]:
                        if conn.laddr and conn.raddr:
                            log_line = f"{time.time()} {conn.laddr.ip} {conn.laddr.port} {conn.raddr.ip} {conn.raddr.port}"
                            connections.append(log_line)

                    
            memory_maps = process.get("memory_maps", [])
            mapped_filepaths = []
            if memory_maps:
                mapped_filepaths = [path.path for path in memory_maps]

            process_data.append({"Process ID": process["pid"], "Name": process["name"], "Username": process["username"],
                                "Status": process["status"], "Executable Path": process["exe"], "Command": cmd_line,
                                "Parent ID": process["ppid"], "Creation Time": creation_time, "Open Files": open_files_str, "Connections": "\r\n".join(connections), "Mapped Filepaths": ",".join(mapped_filepaths)
                                })
        return process_data

    def dict_to_json(self, rows: List[dict]) -> str:
        """Takes a list of rows/dict and returns as a json with a CadoJsonTable header

        :param rows: The List[Dict] of row data e.g. [{'filepath': 'file.txt'}]

        :return: The Json string
        """
        table_dict = {"format": "CadoJsonTable", "rows": rows}
        return json.dumps(table_dict, sort_keys=False, indent=1)

    def get_machine_name(self) -> str:
        """Return machine name without any special characters removed

        :return: The machine name without any special characters
        """
        return remove_special_characters(socket.gethostname())

    def take_screenshot(self) -> Optional[bytes]:
        """Takes a screenshot of all connected monitors and returns the bytes of the image

        :return:  The raw image
        """
        try:
            with mss.mss() as sct:
                monitor = sct.monitors[0] # monitors[0] is all connected monitors in one
                sct_img = sct.grab(monitor)
                png = mss.tools.to_png(sct_img.rgb, sct_img.size)
                return png
        except mss.exception.ScreenShotError:
            logging.error("Unable to take screenshot")
        return None

    def acquire_volatile(self, output_path: Optional[str] = None) -> str:
        """Acquire volatile data into a zip file
        This is called by all OS's

        :return: The filepath of the zip
        """
        self.process_info = self.get_processes()
        self.network_log = self.get_network()
        self.dumped_files = self.dump_loaded_files() if self.include_open else []
        table_data = {}
        table_data["processes"] = self.dict_to_json(self.process_info)
        open_files_dict = [{"Open File": open_file} for open_file in self.dumped_files]
        table_data["open_files"] = self.dict_to_json(open_files_dict)
        if self.take_screenshot:
            screenshot = self.take_screenshot()
        else:
            screenshot = None
        if not output_path:
            output_path = os.path.join("", f"{self.get_machine_name()}-{self.timestamp}.zip")
        # strip .zip if in filename as shutil appends to end
        archive_out =  output_path + ".zip" if not output_path.endswith(".zip") else output_path
        self.output_path = output_path
        with zipfile.ZipFile(archive_out, 'a', compression=zipfile.ZIP_DEFLATED) as zip_file:
            if screenshot:
                zip_file.writestr(f"{self.get_machine_name()}-{self.timestamp}.png", screenshot) 
            for key, value in table_data.items():
                with zip_file.open(f"{key}.json", 'w') as json_file:
                    json_file.write(value.encode())
            if self.network_log:
                logging.info("Adding Netstat Data")
                with zip_file.open("netstat.log", 'w') as network_file:
                    network_file.write("\r\n".join(self.network_log).encode())
            if self.dump_loaded_files:
                for file_path in self.dumped_files:
                    logging.info(f"Adding open file {file_path}")
                    try:
                        if os.path.getsize(file_path) > _MAX_OPEN_FILE_SIZE:
                            logging.warning(f"Skipping file as too large {file_path}")
                        else:
                            try:
                                zip_file.write(file_path, strip_drive(file_path))
                            except PermissionError:
                                logging.warn(f"Permission denied copying {file_path}")
                    except FileNotFoundError:
                        logging.warning(f"Could not open {file_path} for reading")

        return archive_out



from typing import Any

from varc_core.systems.base_system import BaseSystem


class OsxSystem(BaseSystem):

    def __init__(
        self,
        include_memory: bool = True,
        include_open: bool = True,
        extract_dumps: bool = False,
        **kwargs: Any
    ) -> None:
        super().__init__(include_memory=include_memory, include_open=include_open, extract_dumps=extract_dumps, **kwargs)



import zipfile
import tempfile
from tqdm import tqdm
import logging
from typing import Tuple, Optional, Any
from sys import platform
from os import remove as del_file
from os import sep
from pathlib import Path
from varc_core.systems.base_system import BaseSystem


if platform == "win32": # dont try to import on linux
   from sys import maxsize
   import pymem

class WindowsSystem(BaseSystem):
    """
    """

    def __init__(
        self,
        include_memory: bool,
        include_open: bool,
        extract_dumps: bool,
        **kwargs: Any
    ) -> None:
        super().__init__(include_memory=include_memory, include_open=include_open, extract_dumps=extract_dumps, **kwargs)
        if self.include_memory:
            self.dump_processes()
            if self.extract_dumps:
                from varc_core.utils import dumpfile_extraction
                dumpfile_extraction.extract_dumps(Path(self.output_path))

    def read_process(self, handle: int, address: int) -> Tuple[Optional[bytes], int]:
        """ Read a process. Based on pymems pattern module

        :param handle: int of the handle
        :param address: int of the address

        :return: The process contents
        :rtype: Tuple[Optional[bytes], int]
        """

        mbi = pymem.memory.virtual_query(handle, address)
        next_region: int = int(mbi.BaseAddress + mbi.RegionSize)
        allowed_protections = [
            pymem.ressources.structure.MEMORY_PROTECTION.PAGE_EXECUTE_READ,
            pymem.ressources.structure.MEMORY_PROTECTION.PAGE_EXECUTE_READWRITE,
            pymem.ressources.structure.MEMORY_PROTECTION.PAGE_READWRITE,
            pymem.ressources.structure.MEMORY_PROTECTION.PAGE_READONLY,
        ]
        if mbi.state != pymem.ressources.structure.MEMORY_STATE.MEM_COMMIT or mbi.protect not in allowed_protections:
            return None, next_region 
        try:
            page_bytes = None
            page_bytes = pymem.memory.read_bytes(handle, address, mbi.RegionSize)
        except Exception:
            logging.warning("Failed to read a memory page")
        return page_bytes, next_region

    def dump_processes(self) -> None:
        """
        Based on pymem's 'Pattern' module
        """
        archive_out = self.output_path
        for proc in tqdm(self.process_info, desc="Process dump progess", unit=" procs"):
            pid = proc["Process ID"]
            p_name = proc["Name"]

            # Set upper limits on memory address to read
            user_space_limit = 0x7FFFFFFF0000 if maxsize > 2**32 else 0x7fff0000

            # Open handle to process memory for reading
            try:
                p = pymem.Pymem()
                p.open_process_from_id(pid)
            except (TypeError, pymem.exception.CouldNotOpenProcess):
                logging.warning(f"Could not open process {p_name} (pid {pid}) for reading. Cannot dump this process.")
                continue
            
            # Dump all pages the process virtual address space
            next_region = 0
            with zipfile.ZipFile(archive_out, 'a', compression=zipfile.ZIP_DEFLATED) as zip_file:
                with tempfile.NamedTemporaryFile(mode="w+b", buffering=0, delete=False) as tmpfile:
                    while next_region < user_space_limit:
                        proc_page_bytes, next_region = self.read_process(p.process_handle, next_region)
                        if proc_page_bytes:
                            tmpfile.write(proc_page_bytes)
                    zip_file.write(tmpfile.name, f"process_dumps{sep}{p_name}_{pid}.mem")
                del_file(tmpfile.name)
        logging.info(f"Dumping processing has completed. Output file is located: {archive_out}")



from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


VERSION = '1.0.2'

setup(
    name='varc',
    version=VERSION,
    description='varc Volatile Artifact Collector',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Cado Security',
    author_email='varc@cadosecurity.com',
    url='https://github.com/cado-security/varc',
    download_url=f'https://github.com/cado-security/varc/archive/refs/heads/main.zip',
    py_modules=['varc'],
    install_requires=['psutil', 'mss', 'tqdm', 'pymem'],
    packages=find_packages()
)



import logging
from sys import platform

from varc_core.exceptions import MissingOperatingSystemInfo
from varc_core.systems.base_system import BaseSystem


def acquire_system(
    include_memory: bool = True,
    include_open: bool = True,
    extract_dumps: bool = False
) -> BaseSystem:
    """Returns the either a windows or linux system or osx system

    :return: Returns the system object for the OS
    :rtype WindowsSystem or LinuxSystem or OsxSystem
    """  
    logging.info(f"Operating System is: {platform}")
    if platform == "linux" or platform == "linux2":
        from varc_core.systems.linux import LinuxSystem
        return LinuxSystem(include_memory, include_open, extract_dumps)
    elif platform == "darwin":
        from varc_core.systems.osx import OsxSystem
        return OsxSystem(include_memory, include_open, extract_dumps)
    elif platform == "win32":
        from varc_core.systems.windows import WindowsSystem
        return WindowsSystem(include_memory, include_open, extract_dumps)
    else:
        raise MissingOperatingSystemInfo()


import argparse
import logging

from varc_core.systems import acquire_system


if __name__ == "__main__":
    logging_level = logging.INFO
    logging.basicConfig(
        format='[%(asctime)s]:[%(levelname)s] - %(message)s',
        handlers=[
            logging.FileHandler("varc.log"),
            logging.StreamHandler()
        ],
        level=logging_level
    )

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--skip-memory",
        action="store_false",
        dest="include_memory",
        help="Skip collecting process memory, which can be slow",
    )
    parser.add_argument(
        "--skip-open",
        action="store_false",
        dest="include_open",
        help="Skip collecting open files, which can be slow",
    )
    parser.add_argument(
        "--dump-extract",
        action="store_true",
        dest="extract_dumps",
        help="Extract process memory dumps, which can be slow",
    )
    # Allow other arguments - needed for unittests
    parser.add_argument('args', nargs=argparse.REMAINDER)
    args = parser.parse_args()
    acquire_system(
        include_memory=args.include_memory,
        include_open=args.include_open,
        extract_dumps=args.extract_dumps
    )



from zipfile import ZipFile
import unittest

from varc import acquire_system
from varc_core.systems import BaseSystem

# Test importing as a library

class TestImport(unittest.TestCase):
    system: BaseSystem
    zip_path: str

    @classmethod
    def setUpClass(cls) -> None:
        cls.system = acquire_system()
        cls.zip_path = cls.system.acquire_volatile()

    @classmethod
    def tearDownClass(cls) -> None:
        pass

    def test_get_processes(self) -> None:
        processes = self.system.get_processes()
        self.assertTrue(len(processes) > 0)

    def test_get_network(self) -> None:
        network = self.system.get_network()
        self.assertTrue(len(network) > 0)

    def test_got_files(self) -> None:
        # Check we got atleast 10 files
        with ZipFile(self.zip_path) as z:
            self.assertGreater(len(z.namelist()), 10)
